/**
 * Constants
 * ----------------------------------
 */

var Zero_Through_Nine_SET = "0123456789";
var a_Through_z_SET = "abcdefghijklmnopqrstuvwxyz";
var A_Through_Z_SET = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
var Exotic_Symbols_SET = "*=+%$^{¤}]\"\\|´~'`[:;]";
var Exotic_Characters_SET = "øØæÆÿŸìÌòÒûÛàÁêÊúÚöÖâÂéÉóÓäÄåÅëËùÙýÝíÍïÏüÜôÔîÎèÈáÀ";
var Normal_Symbols_SET = ".,-_()!?@/&#";
var Annoying_Characters_SET = "iIoOlL01";

/**
 * Methods
 * ----------------------------------
 */

// Master method to do the password generation process
function doGeneration() {

    // Check the length is > 0
    var pswLen = parseInt(document.getElementById("pswLen").value);
    if(pswLen <= 0 || isNaN(pswLen)) {
        showMsg("Password length cannot be zero or negative!");
        focusAndSelectTextbox("pswLen");
        return;
    }

    // Check the amount is > 0
    var pswAmount = parseInt(document.getElementById("pswAmount").value);
    if(pswAmount <= 0 || isNaN(pswAmount)) {
        showMsg("Password amount cannot be zero or negative!");
        focusAndSelectTextbox("pswAmount");
        return;
    }

    // Check that there's at least chosen > 0
    if(amountOfCharSetsChosen() == 0) {
        showMsg("You have to choose at least one character set!");
        return;
    }

    // Check is password length is shorter than the amount of rules
    // selected not counting "no annoying chars"
    if(amountOfRulesChosen() > pswLen) {
        showMsg("You cannot choose more rules than the length of the password. This only includes: require low-case, upper-case and digis. Since all of these require at least one character to be present if all of them is selected it means that the length has to be 3 at least not "+pswLen);
        return;
    }

    // Check that there are chosen no rules with according char sets
    // unchecked
    var digits = (!document.getElementById("chbZToN").checked &&
                  document.getElementById("chbReqDigits").checked),
        low = (!document.getElementById("chbaToz").checked &&
               document.getElementById("chbReqLow").checked),
        upper = (!document.getElementById("chbAToZ").checked &&
                 document.getElementById("chbReqUpper").checked);
    if(digits || low || upper) {
        var msg = "You cannot choose a rule without choosing the according character set as well! For instance, checking the \"Require digits\" and not the character set [0-9] means that it will run forever checking to get a digit but there are none in the character set.\n\nIn this case the following mismatched:\n";
        if(digits)
            msg = msg + "'Require digits' needs [0-9]\n";
        if(low)
            msg = msg + "'Require low-case chars' needs [a-z]\n";
        if(upper)
            msg = msg + "'Require upper-case chars' needs [A-Z]\n";                
        showMsg(msg);
        return;
    }
    
    // Put together the chosen character set
    var charSet = "";
    if(document.getElementById("chbZToN").checked)
        charSet = charSet + Zero_Through_Nine_SET;
    if(document.getElementById("chbaToz").checked)
        charSet = charSet + a_Through_z_SET;
    if(document.getElementById("chbAToZ").checked)
        charSet = charSet + A_Through_Z_SET;
    if(document.getElementById("chbExChars").checked)
        charSet = charSet + Exotic_Characters_SET;
    if(document.getElementById("chbNormSym").checked)
        charSet = charSet + Normal_Symbols_SET;
    if(document.getElementById("chbExSym").checked)
        charSet = charSet + Exotic_Symbols_SET;
    
    // Add the custom chars
    charSet = charSet + document.getElementById("customChars").value;

    // If the custom chars are defined and the CheckRowOrigin rule is
    // chosen, it is crucial to stop a loop to check if the
    // customChars abides by the rule
    if(document.getElementById("chbCheckRows").checked) {
        if(pswLen == 1) {
            showMsg("When characters cannot originate from only\none row then the password length cannot be 1 (one)!");
            return;
        }
        else if(!checkRows(charSet)) {
            showMsg("The chosen charset cannot generate passwords that does not only originate from one row of the keyboard! These examples are illegal: qwerty, 123, cvbn etc.");
            return;
        }
    }


    // check that if SingleAppearanceCharRule is selected
    // the charset has to be longer or as long as  the 
    // length of passwords to generate
    if(document.getElementById("chbCharAppearance").checked){
        if(charSet.length < pswLen) {
            showMsg("Your chosen character set is too short!\nIf characters cannot appear more than once then the character set needs to be as long or longer than the length of passwords to generate!");
            return;
        }
    }

    // shuffle the character set randomly before using it
    charSet = shuffle(charSet);
//document.write("shuffle="+charSet);
    var psws = "Password(s) generated:<table border=\"1\"><tr><th>Password</th>"+
        (document.getElementById("chbUseHash").checked ? "<th>Hash</th>" : "")+
        "</tr>";
    
    for(k = 0; k < pswAmount; k++) {
        var psw = "",
            hash = "";

        // make sure it complies by the rules if any is selected
        while(true) {
            psw = generate(pswLen, charSet);
            // Determine if there should be made a hash of the
            // generated password
            if(document.getElementById("chbUseHash").checked) {
                if(document.getElementById("rbHashMd5").checked)
                    hash = hex_md5(psw);
                else if(document.getElementById("rbHashSha1").checked)
                    hash = hex_sha1(psw);
                else if(document.getElementById("rbHashSha256").checked)
                    hash = hex_sha256(psw);                
            }
            
            var invalid = false;

            // (If checked) make sure that there's no annoying chars
            if(document.getElementById("chbNoAnn").checked)
                invalid = !checkNoAnnoyingChars(psw);

            if(invalid) continue;
            
            // (If checked) make sure that there's at least one low-case char
            if(document.getElementById("chbReqLow").checked)
                invalid = !checkLowCaseChars(psw);

            if(invalid) continue;
            
            // (If checked) make sure that there's at least one upper-case char
            if(document.getElementById("chbReqUpper").checked)
                invalid = !checkUpperCaseChars(psw);

            if(invalid) continue;

            // (If checked) make sure that there's at least one digit
            if(document.getElementById("chbReqDigits").checked)
                invalid = !checkDigits(psw);

            // (If checked) make sure that all chars does not
            // originate from only one row
            if(document.getElementById("chbCheckRows").checked)
                invalid = !checkRows(psw);      

            // (If checked) make sure that chars does not
            // appear more than once
            if(document.getElementById("chbCharAppearance").checked)
                invalid = !checkAppearance(psw);                   

            if(!invalid) break;            
        }
        psws = psws + "<tr><td>"+psw+"</td>";
        if(hash.length > 0)
            psws = psws + "<td>"+hash+"</td>";
        psws = psws + "</tr>";
    }
    
    document.getElementById("genPsw").innerHTML = psws;
}

function focusAndSelectTextbox(id) {
    document.getElementById(id).focus();
    document.getElementById(id).select();
}

function checkNoAnnoyingChars(data) {
    var nLen = Annoying_Characters_SET.length;
    for(i = 0; i < nLen; i++) {
        if(data.indexOf(Annoying_Characters_SET[i]) != -1)
            return false;
    }
    return true;
}

function checkLowCaseChars(data) {
    var aLen = a_Through_z_SET.length;
    for(i = 0; i < aLen; i++) {
        var c = a_Through_z_SET.charAt(i);
        if(data.indexOf(c) != -1)
            return true;
    }
    return false;
}

function checkUpperCaseChars(data) {
    var ALen = A_Through_Z_SET.length;
    for(i = 0; i < ALen; i++) {
        var c = A_Through_Z_SET.charAt(i);
        if(data.indexOf(c) != -1)
            return true;
    }
    return false;
}

function checkDigits(data) {
    var zLen = Zero_Through_Nine_SET.length;
    for(i = 0; i < zLen; i++) {
        var c = Zero_Through_Nine_SET.charAt(i);
        if(data.indexOf(c) != -1)
            return true;
    }
    return false;
}

function checkAppearance(data) {
	if(data.length == 1)
		return true;

	var c;
	for(i = 0; i < data.length; i++) {
		c = data.charAt(i);
		if(countOccurences(data, c) > 1)
			return false;
	}

	return true;
}

function countOccurences(data, ch) {
    var pos = -1, oldpos = -1, count = 0;
    while(true) {
        oldpos = ++pos;
        pos = data.indexOf(ch, pos);
        if(pos != -1)
            count++;
        else 
            pos = oldpos+1;

        if(pos >= data.length)
            break;
    }
    return count;
}

function checkRows(data) {
    var rows1 = "1234567890+§!\"#¤%&/()=?¡@£$½¥{[]}±|´`",
        rows2 = "ûÛêÊúÚöÖåÅéÉóÓèÈëËùÙýÝíÍïÏüÜôÔîÎqwertyuioìÌòÒpåQWERTYÿŸUIOPÅ^~",
        rows3 = "àÁâÂáÀäÄasdfghjklæøASDFGHJKLÆØ*'",
        rows4 = "zxcvbnmZXCVBNM,.-;:_<>\\";
    var counter1 = 0,
        counter2 = 0,
        counter3 = 0,
        counter4 = 0,
        count = 0;

    for(i = 0; i < 4; i++) {
        for(j = 0; j < data.length; j++) {
            var row;
            if(i == 0) row = rows1;
            else if(i == 1) row = rows2;
            else if(i == 2) row = rows3;
            else if(i == 3) row = rows4;
            if(row.indexOf(data.charAt(j)) != -1) {
                if(i == 0) counter1++;
                else if(i == 1) counter2++;
                else if(i == 2) counter3++;
                else if(i == 3) counter4++;
            }
        }
    }

    if(counter1 == 0) count++;
    if(counter2 == 0) count++;
    if(counter3 == 0) count++;
    if(counter4 == 0) count++;

    if(count == 3)
        return false;
    else
        return true;    
}

function amountOfCharSetsChosen() {
    var res = 0;
    if(document.getElementById("chbZToN").checked) res++;
    if(document.getElementById("chbaToz").checked) res++;
    if(document.getElementById("chbAToZ").checked) res++;
    if(document.getElementById("chbExChars").checked) res++;
    if(document.getElementById("chbNormSym").checked) res++;
    if(document.getElementById("chbExSym").checked) res++;
    if(document.getElementById("customChars").value.length > 0) res++;
    return res;
}

// not counting "no annoying chars" and "check rows" in
function amountOfRulesChosen() {
    var res = 0;
    if(document.getElementById("chbReqLow").checked) res++;
    if(document.getElementById("chbReqUpper").checked) res++;
    if(document.getElementById("chbReqDigits").checked) res++;
    return res;
}

// Method to generate a random password of a given length and with a
// given character set
// PRECONDITION: length is verified as > 0
function generate(length, charSet) {
    var res = "";
    for(i = 0; i < length; i++) {
        var pos = rnd(charSet.length);
        res = res + charSet.charAt(pos);
    }
    return res;
}

// Method to generate a random number from 0 to maxVal
function rnd(maxVal) {
    return Math.floor(Math.random()*maxVal);
}

// Method to shuffle data randomly
function shuffle(data) {
    var len = data.length,
        res = data;
    for(i = 0; i < len; i++) {
        var pos1 = rnd(len),
            pos2 = rnd(len),
            str1 = res.charAt(pos1)+"",
            str2 = res.charAt(pos2)+"",
            res2 = "";
            
        for(j = 0; j < len; j++) {
            if(j == pos1)
                res2 = res2 + str2;
            else if(j == pos2)
                res2 = res2 + str1;
            else
                res2 = res2 + res.charAt(j)+"";
        }
        res = res2;
    }
    return res;
}

// Method to toggle the preset Custom if any of the charsets or the
// length of the password is changed
function toggleCustom() {
    document.getElementById("chbPresetCustom").checked = true;
}

// Method to setup defined presets for password generation
function changePreset(num) {
    switch(num) {
    case 0: // Medium
        document.getElementById("pswLen").value = "8";
        document.getElementById("chbZToN").checked = false;
        document.getElementById("chbaToz").checked = true;
        document.getElementById("chbAToZ").checked = true;
        document.getElementById("chbExChars").checked = false;
        document.getElementById("chbNormSym").checked = false;
        document.getElementById("chbExSym").checked = false;
        document.getElementById("chbReqDigits").checked = false;
        document.getElementById("chbCheckRows").checked = true;
        break;
    case 1: // Hard
        document.getElementById("pswLen").value = "12";
        document.getElementById("chbZToN").checked = true;
        document.getElementById("chbaToz").checked = true;
        document.getElementById("chbAToZ").checked = true;
        document.getElementById("chbExChars").checked = false;
        document.getElementById("chbNormSym").checked = false;
        document.getElementById("chbExSym").checked = false;
        document.getElementById("chbCheckRows").checked = true;
        break;
    case 2: // Paranoid
        document.getElementById("pswLen").value = "25";
        document.getElementById("chbZToN").checked = true;
        document.getElementById("chbaToz").checked = true;
        document.getElementById("chbAToZ").checked = true;
        document.getElementById("chbExChars").checked = true;
        document.getElementById("chbNormSym").checked = true;
        document.getElementById("chbExSym").checked = true;
        document.getElementById("chbCheckRows").checked = true;
        break;
    default:
        // Use 'hard' if incorrect input
        changePreset(1);
        break;
    }
}

// Method to switch on/off the radiobuttons for selecting hash
// algorithms
function switchHashes() {
    document.getElementById("rbHashMd5").disabled = !document.getElementById("rbHashMd5").disabled;
    document.getElementById("rbHashSha1").disabled = !document.getElementById("rbHashSha1").disabled;
    document.getElementById("rbHashSha256").disabled = !document.getElementById("rbHashSha256").disabled;
    //document.getElementById("rbHashSha512").disabled = !document.getElementById("rbHashSha512").disabled;
}

// Method to show a message window
function showMsg(message) {
    alert(message);
}