package com.aconiac.apg.gui;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.text.BadLocationException;

import com.aconiac.apg.domain.CheckRowOriginRule;
import com.aconiac.apg.domain.HasHighCaseCharsRule;
import com.aconiac.apg.domain.HasLowCaseCharsRule;
import com.aconiac.apg.domain.HasNumbersRule;
import com.aconiac.apg.domain.HashAlg;
import com.aconiac.apg.domain.Hasher;
import com.aconiac.apg.domain.HasherImpl;
import com.aconiac.apg.domain.NoAnnoyingCharsRule;
import com.aconiac.apg.domain.PasswordGenerator;
import com.aconiac.apg.domain.PasswordGeneratorImpl;
import com.aconiac.apg.domain.Rule;
import com.aconiac.apg.domain.SingleAppearanceCharRule;

/**
 * Enhanced GUI
 * 
 * This GUI is the new an improved GUI with better graphics and 
 * more fine tuned setup
 * 
 * @author Michael Dahl
 * @author Michael Lind Mortensen
 * 
 *
 */
public class MainWindow extends JFrame{
	private static JTabbedPane tabbedPane;
	private static final Dimension appSize = new Dimension(500, 400);
	private static final String HEADER = "/com/aconiac/apg/gfx/header.png";
	private static final String ICON = "/com/aconiac/apg/gfx/icon.png";
	private static final String VERSION_STRING = "APG_1.2";
	
	// Settings tab / panel
	private static JPanel panelSettings;
	private static int panelSettingsIndex;
	private static String panelSettingsTitle = "Settings";
	private static String panelSettingsTooltip = "Change overall settings for password generation";
	private static int panelSettingsMnemonic = KeyEvent.VK_S;
	private static JRadioButton rbtnPresetMedium;
    private static JRadioButton rbtnPresetHard;
    private static JRadioButton rbtnPresetParanoid;
    private static JRadioButton rbtnPresetCustom;
    private static ButtonGroup presetBtnGroup;
	private static final int SLIDERLENGTHMIN = 1,
	    SLIDERLENGTHMAX = 35,
	    SLIDERLENGTHINIT = 12,
	    SLIDERAMOUNTMIN = 1,
	    SLIDERAMOUNTMAX = 150,
	    SLIDERAMOUNTINIT = 1;
	private static JSlider sliderPasswordLength;
	private static boolean sliderPasswordLengthIgnoreNotifications = false;
	private static JSlider sliderPasswordAmount;
	private static boolean sliderPasswordAmountIgnoreNotifications = false;
	private static JTextField fieldPasswordLength;
	private static boolean fieldPasswordLengthIgnoreNotifications = false;
	private static JTextField fieldPasswordAmount;
	private static boolean fieldPasswordAmountIgnoreNotifications = false;
	private static JButton btnPasswordLengthP5;
	private static JButton btnPasswordLengthM5;
	private static JButton btnPasswordAmountP5;
	private static JButton btnPasswordAmountM5;
	private static JButton btnGenerate;
	
	// Advanced tab / panel
	private static JPanel panelAdvanced;
	private static int panelAdvancedIndex;
	private static String panelAdvancedTitle = "Advanced";
	private static String panelAdvancedTooltip = "Change advanced settings for password generation";
	private static int panelAdvancedMnemonic = KeyEvent.VK_A;
	// Characters
	private static JCheckBox chbCharSetZeroToNine;
    private static JCheckBox chbCharSetaToz;
    private static JCheckBox chbCharSetAToZ;
    private static JCheckBox chbCharSetExotic;
    private static JCheckBox chbCharSetNormalSymbols;
    private static JCheckBox chbCharSetExoticSymbols;
    private static JTextField txtExtraEntropy;
    // Rules
    private static JCheckBox chbNoAnnoying;
    private static JCheckBox chbReqLowCase;
    private static JCheckBox chbReqHighCase;
    private static JCheckBox chbReqDigits;
    private static JCheckBox chbCheckRows;
    private static JCheckBox chbCharAppearance;
    // Hashings
    private static JCheckBox chbUseHash;    
    private static JRadioButton rbtnHashMD5;
    private static JRadioButton rbtnHashSHA1;
    private static JRadioButton rbtnHashSHA256;
    private static JRadioButton rbtnHashSHA512;  
    private static ButtonGroup hashGroup;
	
	// Output tab / panel
	private static JPanel panelOutput;
	private static int panelOutputIndex;
	private static String panelOutputTitle = "Output";
	private static String panelOutputTooltip = "List of generated passwords";
	private static int panelOutputMnemonic = KeyEvent.VK_O;
	private static Vector<String> vectorPass = new Vector<String>();
	private static Vector<String> vectorHash = new Vector<String>();
	private static JButton btnOutputCopy;
	private static JButton btnOutputSave;
	
	/**
	 * Main method of gui
	 */
	public static void main(String[] args) {
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				MainWindow gui = new MainWindow();
			}
		});
	}
	
	/**
	 * Constructor for the main window, will construct the entire GUI and
	 * ensure all listeners are added.
	 */
	public MainWindow() {
		// Frame properties
		this.setTitle("Aconiac Password Generator");
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.setSize(appSize);
		this.setResizable(false);
		this.setVisible(true);
		this.setLayout(new BorderLayout());
		this.setIconImage((getClass().getResource(ICON)==null?new ImageIcon(ICON).getImage():new ImageIcon(getClass().getResource(ICON)).getImage()));
		
		// Center window
		Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
		this.setLocation((int)(screenSize.getWidth()-appSize.getWidth())/2, (int)(screenSize.getHeight()-appSize.getHeight())/2);
		
		// Primary banner/header
		JPanel panelHeader = new JPanel();
		panelHeader.setLayout(new BorderLayout());
		JLabel lblHeader = new JLabel(getClass().getResource(HEADER)==null?new ImageIcon(HEADER):new ImageIcon(getClass().getResource(HEADER)));
		//panelHeader.add(new JLabel("A Header will be placed here"));
		panelHeader.add(lblHeader, BorderLayout.CENTER);
		panelHeader.setPreferredSize(new Dimension((int)appSize.getWidth(), 75));
		this.add(panelHeader, BorderLayout.NORTH);
		
		/*
		 * TABBED PANE
		 */
		tabbedPane = new JTabbedPane();
		tabbedPane.setTabPlacement(JTabbedPane.TOP);
			
			/*
			 * SETTINGS TAB
			 */
			panelSettings = new JPanel();
			panelSettings.setLayout(new BoxLayout(panelSettings, BoxLayout.PAGE_AXIS));
			
				// Preset panel
				JPanel psPreset = new JPanel();
				psPreset.setLayout(new BoxLayout(psPreset, BoxLayout.LINE_AXIS));
				psPreset.setBorder(BorderFactory.createTitledBorder(
						BorderFactory.createLineBorder(Color.GRAY, 1), "Presets"));
				psPreset.setPreferredSize(new Dimension((int)appSize.getWidth(),0));
				psPreset.setMaximumSize(new Dimension((int)appSize.getWidth(),45));
				// Create radio buttons
				rbtnPresetMedium = new JRadioButton("Medium");
		        rbtnPresetHard = new JRadioButton("Strong", true);
		        rbtnPresetParanoid = new JRadioButton("Paranoid");
		        rbtnPresetCustom = new JRadioButton("Custom");
		        // Create radio button group
		        presetBtnGroup = new ButtonGroup();
		        presetBtnGroup.add(rbtnPresetMedium);
		        presetBtnGroup.add(rbtnPresetHard);
		        presetBtnGroup.add(rbtnPresetParanoid);
		        presetBtnGroup.add(rbtnPresetCustom);
				// Add to panel
		        psPreset.add(rbtnPresetMedium);
		        psPreset.add(rbtnPresetHard);
		        psPreset.add(rbtnPresetParanoid);
		        psPreset.add(rbtnPresetCustom);
		        // Add to tab
				panelSettings.add(psPreset);
				
				// Settings/Length panel
				JPanel psLength = new JPanel();
				GridBagLayout psG = new GridBagLayout();
				GridBagConstraints psC = new GridBagConstraints();
				psLength.setLayout(psG);
				psC.fill = GridBagConstraints.BOTH;
				psLength.setBorder(BorderFactory.createTitledBorder(
						BorderFactory.createLineBorder(Color.GRAY, 1), "Password settings"));
				psLength.setPreferredSize(new Dimension((int)appSize.getWidth(), 0));
				psLength.setMaximumSize(new Dimension((int)appSize.getWidth(), 100));
				// First line, length
				JLabel lblLength = new JLabel("Length");
				psC.weightx = 1.0;	psG.setConstraints(lblLength, psC);	psLength.add(lblLength);
				sliderPasswordLength = new JSlider(SLIDERLENGTHMIN, SLIDERLENGTHMAX, SLIDERLENGTHINIT);
				psC.weightx = 1.0;	psG.setConstraints(sliderPasswordLength, psC);	psLength.add(sliderPasswordLength);
				btnPasswordLengthP5 = new JButton("+5");
				psC.weightx = 0.25;	psG.setConstraints(btnPasswordLengthP5, psC);	psLength.add(btnPasswordLengthP5);
				btnPasswordLengthM5 = new JButton("-5");
				psC.weightx = 0.25;	psG.setConstraints(btnPasswordLengthM5, psC);	psLength.add(btnPasswordLengthM5);	
				fieldPasswordLength = new JTextField(""+SLIDERLENGTHINIT);
				fieldPasswordLength.setPreferredSize(new Dimension(50, 20));
				psC.gridwidth = GridBagConstraints.REMAINDER;	// end row
				psC.weightx = 1.0;	psG.setConstraints(fieldPasswordLength, psC);	psLength.add(fieldPasswordLength);
				psC.gridwidth = 1;
				// Second line, amount
				JLabel lblAmount = new JLabel("Amount");
				psC.weightx = 1.0;	psG.setConstraints(lblAmount, psC);	psLength.add(lblAmount);
				sliderPasswordAmount = new JSlider(SLIDERAMOUNTMIN, SLIDERAMOUNTMAX, SLIDERAMOUNTINIT);
				psC.weightx = 1.0;	psG.setConstraints(sliderPasswordAmount, psC);	psLength.add(sliderPasswordAmount);
				btnPasswordAmountP5 = new JButton("+5");
				psC.weightx = 0.25;	psG.setConstraints(btnPasswordAmountP5, psC);	psLength.add(btnPasswordAmountP5);
				btnPasswordAmountM5 = new JButton("-5");
				psC.weightx = 0.25;	psG.setConstraints(btnPasswordAmountM5, psC);	psLength.add(btnPasswordAmountM5);
				fieldPasswordAmount = new JTextField(""+SLIDERAMOUNTINIT);
				fieldPasswordAmount.setPreferredSize(new Dimension(50 ,20));
				psC.gridwidth = GridBagConstraints.REMAINDER;
				psC.weightx = 1.0;	psG.setConstraints(fieldPasswordAmount, psC);	psLength.add(fieldPasswordAmount);
				psC.gridwidth = 1;
				// Add it to the tab
				panelSettings.add(psLength);
				// Generate button
				JPanel psGenerate = new JPanel();
				psGenerate.setLayout(new BoxLayout(psGenerate, BoxLayout.LINE_AXIS));
				// Label
				JLabel lblGenerate = new JLabel("Proceed to generate password(s)   ");
				psGenerate.add(lblGenerate, BorderLayout.CENTER);
				// Button
				btnGenerate = new JButton("Generate");
				psGenerate.add(btnGenerate, BorderLayout.EAST);
				// Add it to the tab
				panelSettings.add(psGenerate);
				
			// Add it to pane
			tabbedPane.addTab(panelSettingsTitle, panelSettings);
			panelSettingsIndex = tabbedPane.indexOfComponent(panelSettings);
			tabbedPane.setToolTipTextAt(panelSettingsIndex, panelSettingsTooltip);
			tabbedPane.setMnemonicAt(panelSettingsIndex, panelSettingsMnemonic);
			
			
			/*
			 * ADVANCED TAB
			 */
			panelAdvanced = new JPanel();
			panelAdvanced.setLayout(new BorderLayout());
				// Characters panel
				JPanel paChar = new JPanel();
				paChar.setBorder(BorderFactory.createTitledBorder(
						BorderFactory.createLineBorder(Color.GRAY, 1), "Character sets"));
				paChar.setLayout(new BoxLayout(paChar, BoxLayout.PAGE_AXIS));
				// initialize checkboxes
				chbCharSetZeroToNine = new JCheckBox("[0-9]");
		        chbCharSetaToz = new JCheckBox("[a-z]");
		        chbCharSetAToZ = new JCheckBox("[A-Z]");
		        chbCharSetExotic = new JCheckBox("Exotic chars");
		        chbCharSetNormalSymbols = new JCheckBox("[.,-_)!?@(/&#]");
		        chbCharSetExoticSymbols = new JCheckBox("Exotic symbols");
		        txtExtraEntropy = new JTextField(10);
		        txtExtraEntropy.setMaximumSize(new Dimension((int)appSize.getWidth(), 23));
		        paChar.add(chbCharSetZeroToNine);
		        paChar.add(chbCharSetaToz);
		        paChar.add(chbCharSetAToZ);
		        paChar.add(chbCharSetExotic);
		        paChar.add(chbCharSetNormalSymbols);
		        paChar.add(chbCharSetExoticSymbols);
		        paChar.add(new JLabel("Custom chars"));
		        paChar.add(txtExtraEntropy);
		        // default is 'hard' preset
		        chbCharSetZeroToNine.setSelected(true);
		        chbCharSetaToz.setSelected(true);
		        chbCharSetAToZ.setSelected(true);
		        chbCharSetNormalSymbols.setSelected(false);
		        chbCharSetExoticSymbols.setSelected(false);
		        chbCharSetExotic.setSelected(false);
		        // Add to tab
		        panelAdvanced.add(paChar, BorderLayout.WEST);
		        // Rules panel
		        JPanel paRules = new JPanel();
		        paRules.setBorder(BorderFactory.createTitledBorder(
		        		BorderFactory.createLineBorder(Color.GRAY, 1), "Rules to apply"));
		        paRules.setLayout(new BoxLayout(paRules, BoxLayout.PAGE_AXIS));
		        // initialize
		        chbNoAnnoying = new JCheckBox("No annoying chars (i,I,o,O,l,L,1)");
		        chbReqLowCase = new JCheckBox("Require low-case chars");
		        chbReqHighCase = new JCheckBox("Require high-case chars");
		        chbReqDigits = new JCheckBox("Require digits");
		        chbCheckRows = new JCheckBox("Chars can't come from only one row");
		        chbCharAppearance = new JCheckBox("Chars can only appear once");
		        paRules.add(chbNoAnnoying);
		        paRules.add(chbReqLowCase);
		        paRules.add(chbReqHighCase);
		        paRules.add(chbReqDigits);
		        paRules.add(chbCheckRows);
		        paRules.add(chbCharAppearance);
		        // .. and with all rules on
		        chbReqDigits.setSelected(true);
		        chbReqLowCase.setSelected(true);
		        chbReqHighCase.setSelected(true);
		        chbNoAnnoying.setSelected(true);
		        chbCheckRows.setSelected(true);
		        // Add to pane
		        panelAdvanced.add(paRules, BorderLayout.CENTER);
		        // Hashings
		        JPanel paHash = new JPanel();
		        paHash.setBorder(BorderFactory.createTitledBorder(
		        		BorderFactory.createLineBorder(Color.GRAY, 1), "Hash algorithms"));
		        paHash.setPreferredSize(new Dimension(110,0));
		        paHash.setLayout(new BoxLayout(paHash, BoxLayout.PAGE_AXIS));
		        // initialize
		        chbUseHash = new JCheckBox("Use one?");
		        rbtnHashMD5 = new JRadioButton("MD5");
		        rbtnHashSHA1 = new JRadioButton("SHA1");
		        rbtnHashSHA256 = new JRadioButton("SHA256");
		        rbtnHashSHA512 = new JRadioButton("SHA512");
		        // groupings
		        hashGroup = new ButtonGroup();
		        hashGroup.add(rbtnHashMD5);
		        hashGroup.add(rbtnHashSHA1);
		        hashGroup.add(rbtnHashSHA256);
		        hashGroup.add(rbtnHashSHA512);
		        paHash.add(chbUseHash);
		        paHash.add(rbtnHashMD5);
		        paHash.add(rbtnHashSHA1);
		        paHash.add(rbtnHashSHA256);
		        paHash.add(rbtnHashSHA512);
		        // Inital values
		        chbUseHash.setSelected(false);
		        rbtnHashMD5.setSelected(true);
		        rbtnHashMD5.setEnabled(false);
		        rbtnHashSHA1.setEnabled(false);
		        rbtnHashSHA256.setEnabled(false);
		        rbtnHashSHA512.setEnabled(false);
		        // Add to tab
		        panelAdvanced.add(paHash, BorderLayout.EAST);
		        
			// Add it to pane
			tabbedPane.addTab(panelAdvancedTitle, panelAdvanced);
			panelAdvancedIndex = tabbedPane.indexOfComponent(panelAdvanced);
			tabbedPane.setToolTipTextAt(panelAdvancedIndex, panelAdvancedTooltip);
			tabbedPane.setMnemonicAt(panelAdvancedIndex, panelAdvancedMnemonic);
			
			/*
			 * OUTPUT TAB
			 */
			panelOutput = new JPanel();
			panelOutput.setLayout(new BorderLayout());
			// Add it to pane
			tabbedPane.addTab(panelOutputTitle, panelOutput);
			panelOutputIndex = tabbedPane.indexOfComponent(panelOutput);
			tabbedPane.setToolTipTextAt(panelOutputIndex, panelOutputTooltip);
			tabbedPane.setMnemonicAt(panelOutputIndex, panelOutputMnemonic);
			tabbedPane.setEnabledAt(panelOutputIndex, false);
			
			
		// Add tabs-pane to frame
		this.add(tabbedPane, BorderLayout.CENTER);
		
		
		/*
		 * ACTION LISTENERS
		 */
		// Password Length incrementor/decrementor
		sliderPasswordLength.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent evt) {
				if(!sliderPasswordLengthIgnoreNotifications) {
					fieldPasswordLengthIgnoreNotifications = true;
					changeField(fieldPasswordLength, sliderPasswordLength, ChangeOperator.EXACT, sliderPasswordLength.getValue());
					fieldPasswordLengthIgnoreNotifications = false;
				}
			}
		});
		btnPasswordLengthP5.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if(!sliderPasswordLengthIgnoreNotifications) {
					fieldPasswordLengthIgnoreNotifications = true;
					changeField(fieldPasswordLength, sliderPasswordLength, ChangeOperator.PLUS, 5);
					fieldPasswordLengthIgnoreNotifications = false;
				}
			}
		});
		btnPasswordLengthM5.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if(!sliderPasswordLengthIgnoreNotifications) {
					fieldPasswordLengthIgnoreNotifications = true;
					changeField(fieldPasswordLength, sliderPasswordLength, ChangeOperator.MINUS, 5);
					fieldPasswordLengthIgnoreNotifications = false;
				}
			}
		});
		fieldPasswordLength.getDocument().addDocumentListener(new DocumentListener() {
			public void removeUpdate(DocumentEvent e) {
				if(!fieldPasswordLengthIgnoreNotifications) {
					try {
						/*
						 * Ensures no changes to field are made when this call
						 * fires off a actionPerformed event to all the other
						 * listeners.
						 */
						sliderPasswordLengthIgnoreNotifications = true;
						// Ignore rules applied, check enforced when focus is lost instead
						changeSlider(sliderPasswordLength, e.getDocument().getText(0, e.getDocument().getLength()), true);
						sliderPasswordLengthIgnoreNotifications = false;
					} catch(BadLocationException e2) {
						// Shouln't happen
					}
				}
			}
			
			public void insertUpdate(DocumentEvent e) {
				if(!fieldPasswordLengthIgnoreNotifications) {
					try {
						/*
						 * Ensures no changes to field are made when this call
						 * fires off a actionPerformed event to all the other
						 * listeners.
						 */
						sliderPasswordLengthIgnoreNotifications = true;
						// Ignore rules applied, check enforced when focus is lost instead
						changeSlider(sliderPasswordLength, e.getDocument().getText(0, e.getDocument().getLength()), true);
						sliderPasswordLengthIgnoreNotifications = false;
					} catch(BadLocationException e2) {
						// Shouln't happen
					}
				}
			}
			
			public void changedUpdate(DocumentEvent e) {
				if(!fieldPasswordLengthIgnoreNotifications) {
					try {
						/*
						 * Ensures no changes to field are made when this call
						 * fires off a actionPerformed event to all the other
						 * listeners.
						 */
						sliderPasswordLengthIgnoreNotifications = true;
						// Ignore rules applied, check enforced when focus is lost instead
						changeSlider(sliderPasswordLength, e.getDocument().getText(0, e.getDocument().getLength()), true);
						sliderPasswordLengthIgnoreNotifications = false;
					} catch(BadLocationException e2) {
						// Shouln't happen
					}
				}
			}
		});
		fieldPasswordLength.addFocusListener(new FocusListener() {
			public void focusLost(FocusEvent e) {
				// Check rules
				/*
				 * Ensures no changes to field are made when this call
				 * fires off a actionPerformed event to all the other
				 * listeners.
				 */
				sliderPasswordLengthIgnoreNotifications = true;
				// Ignore rules applied, check enforced when focus is lost instead
				changeSlider(sliderPasswordLength, fieldPasswordLength.getText());
				sliderPasswordLengthIgnoreNotifications = false;
			}
			public void focusGained(FocusEvent e) {
				// Nothing to do here
			}
		});
		// Password Amount incrementor/decrementor
		sliderPasswordAmount.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent evt) {
				if(!sliderPasswordAmountIgnoreNotifications) {
					fieldPasswordAmountIgnoreNotifications = true;
					changeField(fieldPasswordAmount, sliderPasswordAmount, ChangeOperator.EXACT, sliderPasswordAmount.getValue());
					fieldPasswordAmountIgnoreNotifications = false;
				}
			}
		});
		btnPasswordAmountP5.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if(!sliderPasswordAmountIgnoreNotifications) {
					fieldPasswordAmountIgnoreNotifications = true;
					changeField(fieldPasswordAmount, sliderPasswordAmount, ChangeOperator.PLUS, 5);
					fieldPasswordAmountIgnoreNotifications = false;
				}
			}
		});
		btnPasswordAmountM5.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				if(!sliderPasswordAmountIgnoreNotifications) {
					fieldPasswordAmountIgnoreNotifications = true;
					changeField(fieldPasswordAmount, sliderPasswordAmount, ChangeOperator.MINUS, 5);
					fieldPasswordAmountIgnoreNotifications = false;
				}
			}
		});
		fieldPasswordAmount.getDocument().addDocumentListener(new DocumentListener() {
			public void removeUpdate(DocumentEvent e) {
				if(!fieldPasswordAmountIgnoreNotifications) {
					try {
						sliderPasswordAmountIgnoreNotifications = true;
						// Ignore rules applied, check enforced when focus is lost instead
						changeSlider(sliderPasswordAmount, e.getDocument().getText(0, e.getDocument().getLength()), true);
						sliderPasswordAmountIgnoreNotifications = false;
					} catch(BadLocationException evt) {
						// Shouldn't happen
					}
				}
			}
			
			public void insertUpdate(DocumentEvent e) {
				if(!fieldPasswordAmountIgnoreNotifications) {
					try {
						sliderPasswordAmountIgnoreNotifications = true;
						// Ignore rules applied, check enforced when focus is lost instead
						changeSlider(sliderPasswordAmount, e.getDocument().getText(0, e.getDocument().getLength()), true);
						sliderPasswordAmountIgnoreNotifications = false;
					} catch(BadLocationException evt) {
						// Shouldn't happen
					}
				}
			}
			
			public void changedUpdate(DocumentEvent e) {
				if(!fieldPasswordAmountIgnoreNotifications) {
					try {
						sliderPasswordAmountIgnoreNotifications = true;
						// Ignore rules applied, check enforced when focus is lost instead
						changeSlider(sliderPasswordAmount, e.getDocument().getText(0, e.getDocument().getLength()), true);
						sliderPasswordAmountIgnoreNotifications = false;
					} catch(BadLocationException evt) {
						// Shouldn't happen
					}
				}
			}
		
		});
		fieldPasswordAmount.addFocusListener(new FocusListener() {
			public void focusLost(FocusEvent e) {
				// Enforce rule check
				sliderPasswordAmountIgnoreNotifications = true;
				changeSlider(sliderPasswordAmount, fieldPasswordAmount.getText());
				sliderPasswordAmountIgnoreNotifications = false;
			}
			public void focusGained(FocusEvent e) {
				// Nothing to do here
			}
		});
		// Hash algorithm
		chbUseHash.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                JCheckBox box = (JCheckBox) event.getSource();
                if(box.isSelected()) {
                    rbtnHashMD5.setEnabled(true);
                    rbtnHashSHA1.setEnabled(true);
                    rbtnHashSHA256.setEnabled(true);
                    rbtnHashSHA512.setEnabled(true); 
                } else {
                    rbtnHashMD5.setEnabled(false);
                    rbtnHashSHA1.setEnabled(false);
                    rbtnHashSHA256.setEnabled(false);
                    rbtnHashSHA512.setEnabled(false);
                }
            }
        });
		// Characters
		chbCharSetZeroToNine.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                JCheckBox box = (JCheckBox) event.getSource();
                rbtnPresetCustom.setSelected(true);
                if(!box.isSelected())
                    chbReqDigits.setSelected(false);
            }
        });
	    chbCharSetaToz.addActionListener(new ActionListener() {
	            public void actionPerformed(ActionEvent event) {
	                JCheckBox box = (JCheckBox) event.getSource();
	                rbtnPresetCustom.setSelected(true);
	                if(!box.isSelected())
	                    chbReqLowCase.setSelected(false);                    
	            }
	        });
	    chbCharSetAToZ.addActionListener(new ActionListener() {
	            public void actionPerformed(ActionEvent event) {
	                JCheckBox box = (JCheckBox) event.getSource();
	                rbtnPresetCustom.setSelected(true);
	                if(!box.isSelected())
	                    chbReqHighCase.setSelected(false);
	            }
	        });
	    chbCharSetExoticSymbols.addActionListener(new ActionListener() {
	            public void actionPerformed(ActionEvent event) {
	                rbtnPresetCustom.setSelected(true);
	            }
	        });
	    chbCharSetExotic.addActionListener(new ActionListener() {
	            public void actionPerformed(ActionEvent event) {
	                rbtnPresetCustom.setSelected(true);
	            }
	        });
	    chbCharSetNormalSymbols.addActionListener(new ActionListener() {
	            public void actionPerformed(ActionEvent event) {
	                rbtnPresetCustom.setSelected(true);
	            }
	        });
	    // Rules
	    chbReqDigits.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                rulesCheck(chbReqDigits, chbCharSetZeroToNine);
            }
        });
	    chbReqLowCase.addActionListener(new ActionListener() {
	            public void actionPerformed(ActionEvent event) {
	                rulesCheck(chbReqLowCase, chbCharSetaToz);                    
	            }
	        });
	    chbReqHighCase.addActionListener(new ActionListener() {
	            public void actionPerformed(ActionEvent event) {
	                rulesCheck(chbReqHighCase, chbCharSetAToZ);                    
	            }
	        });
	    // Presets
	    rbtnPresetMedium.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                chbCharSetZeroToNine.setSelected(true);
                chbCharSetaToz.setSelected(true);
                chbCharSetAToZ.setSelected(true);
                chbCharSetNormalSymbols.setSelected(false);
                chbCharSetExoticSymbols.setSelected(false);
                chbCharSetExotic.setSelected(false);
                
                chbReqDigits.setSelected(false);
                
                changeField(fieldPasswordLength, sliderPasswordLength, ChangeOperator.EXACT, 10);
            }
        });
	    rbtnPresetHard.addActionListener(new ActionListener() {
	            public void actionPerformed(ActionEvent event) {
	                chbCharSetZeroToNine.setSelected(true);
	                chbCharSetaToz.setSelected(true);
	                chbCharSetAToZ.setSelected(true);
	                chbCharSetNormalSymbols.setSelected(false);
	                chbCharSetExoticSymbols.setSelected(false);
	                chbCharSetExotic.setSelected(false);
	
	                changeField(fieldPasswordLength, sliderPasswordLength, ChangeOperator.EXACT, 16);
	            }
	        });
	    rbtnPresetParanoid.addActionListener(new ActionListener() {
	            public void actionPerformed(ActionEvent event) {
	                chbCharSetZeroToNine.setSelected(true);
	                chbCharSetaToz.setSelected(true);
	                chbCharSetAToZ.setSelected(true);
	                chbCharSetNormalSymbols.setSelected(true);
	                chbCharSetExoticSymbols.setSelected(true);
	                chbCharSetExotic.setSelected(true);
	
	                changeField(fieldPasswordLength, sliderPasswordLength, ChangeOperator.EXACT, 35);
	            }
	        });
	    /*
	     * GENERATE BUTTON LISTENER
	     */
	    btnGenerate.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                PasswordGenerator gen = new PasswordGeneratorImpl();
                Hasher hasher = new HasherImpl();
                
                vectorPass = new Vector<String>();
                vectorHash = new Vector<String>();

                // determine if a character-set were given and use them
                int pswLen;
                try {
                	pswLen = Integer.valueOf(fieldPasswordLength.getText());
                } catch(NumberFormatException e) {
                	// Update listeners, should ensure this doesn't happen
                	return;
                }

                // verify amount of passwords to generate is a number
                int pswAmount;
                try {
                	pswAmount = Integer.valueOf(fieldPasswordAmount.getText());
                } catch(NumberFormatException e) {
                	// Update listeners should ensure this doesn't happen
                	return;
                }
                
                String charSet = "",
                    oldCharSet = gen.getCharacterSet();
                
                if(chbNoAnnoying.isSelected())
                    gen.addRule(new NoAnnoyingCharsRule());
                if(chbReqLowCase.isSelected())
                    gen.addRule(new HasLowCaseCharsRule());
                if(chbReqHighCase.isSelected())
                    gen.addRule(new HasHighCaseCharsRule());
                if(chbReqDigits.isSelected())
                    gen.addRule(new HasNumbersRule());
                if(chbCheckRows.isSelected())
                    gen.addRule(new CheckRowOriginRule());                    
                if(chbCharAppearance.isSelected())
                    gen.addRule(new SingleAppearanceCharRule());                    
                if(chbCharSetZeroToNine.isSelected())
                    charSet += PasswordGeneratorImpl.Zero_Through_Nine_SET;
                if(chbCharSetaToz.isSelected())
                    charSet += PasswordGeneratorImpl.a_Through_z_SET;
                if(chbCharSetAToZ.isSelected())
                    charSet += PasswordGeneratorImpl.A_Through_Z_SET;
                if(chbCharSetExotic.isSelected())
                    charSet += PasswordGeneratorImpl.Exotic_Characters_SET;
                if(chbCharSetNormalSymbols.isSelected())
                    charSet += PasswordGeneratorImpl.Normal_Symbols_SET;
                if(chbCharSetExoticSymbols.isSelected())
                    charSet += PasswordGeneratorImpl.Exotic_Symbols_SET;

                String customChar = txtExtraEntropy.getText();
                charSet += customChar;
                
                // if no options for character sets were set then use default
                if(charSet.equals("")) gen.setCharacterSet(oldCharSet);
                // else use them
                else gen.setCharacterSet(charSet);

                // check charset for row rule
                if(chbCheckRows.isSelected()) {
                    Rule r = new CheckRowOriginRule();
                    if(!r.isValid(gen.getCharacterSet())) {
                        showUsage(5);
                        return;
                    }
                }

                // check that if SingleAppearanceCharRule is selected
                // the charset has to be longer or as long as  the 
                // length of passwords to generate
                if(chbCharAppearance.isSelected()) {
                    if(gen.getCharacterSet().length() < pswLen) {
                        showUsage(6);
                        return;
                    }
                }

                // should there be generated hashes as well?
                boolean useHashes = false;
                HashAlg alg = null;
                if(chbUseHash.isSelected()) {
                    useHashes = true;
                    if(rbtnHashMD5.isSelected())
                        alg = HashAlg.MD5;
                    else if(rbtnHashSHA1.isSelected())
                        alg = HashAlg.SHA1;
                    else if(rbtnHashSHA256.isSelected())
                        alg = HashAlg.SHA256;
                    else if(rbtnHashSHA512.isSelected())
                        alg = HashAlg.SHA512;                        
                }
                
                // generate password
                ArrayList<String> list = new ArrayList<String>();
                int charSetSize = gen.getCharacterSet().length();
                double entropyBits = gen.checkStrength(pswLen, charSetSize);
                /*String text = "<font size=3 face=verdana>"+
                    "Generating "+pswAmount+" password(s) of length "+pswLen+":<br>"+
                    "Password strength: "+(int)entropyBits+" bits<br>"+
                    "Level og protection: "+gen.strengthDescription(entropyBits)+"<br>" +
                    "<table border=1 cellpadding=2 cellspacing=0><th>Password</th>";*/

                for(int k = 0; k < pswAmount; k++) {
                    String psw = "";

                    // keep generating until a password is found that does not
                    // exist already
                    while(true) {
                        psw = gen.generate(pswLen);
                        if(!list.contains(psw)) {
                            list.add(psw);
                            vectorPass.add(psw);
                            break;
                        }
                    }
                    
                    if(useHashes) {
                        vectorHash.add(hasher.genHexStringHash(psw, alg));
                    }
                }
                
                // Create GUI
                panelOutput.removeAll();
                if (useHashes) {
                	JPanel panelPass = new JPanel();
                	JPanel panelHash = new JPanel();
                	panelPass.setLayout(new BorderLayout());
                	panelHash.setLayout(new BorderLayout());
                	
                	JLabel lblPass = new JLabel(pswAmount > 1 ? "Passwords" : "Password");
                	lblPass.setOpaque(true);
    				lblPass.setBackground(Color.BLACK);
    				lblPass.setForeground(Color.WHITE);
    				panelPass.add(lblPass, BorderLayout.NORTH);
    				
                	JLabel lblHash = new JLabel("Hash (" + alg + ")");
                	lblHash.setOpaque(true);
    				lblHash.setBackground(Color.BLACK);
    				lblHash.setForeground(Color.WHITE);
    				panelHash.add(lblHash, BorderLayout.NORTH);
    				
    				final JList listPass = new JList(vectorPass);
    				listPass.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
    				panelPass.add(listPass, BorderLayout.CENTER);
    				
    				final JList listHash = new JList(vectorHash);
    				listHash.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
    				panelHash.add(listHash, BorderLayout.CENTER);
    				
    				// Make lists update selection on each other
    				listHash.addListSelectionListener(new ListSelectionListener() {
						public void valueChanged(ListSelectionEvent e) {
							JList source = (JList)e.getSource();
							// Update listPass
							listPass.setSelectionInterval(source.getAnchorSelectionIndex(), source.getLeadSelectionIndex());
						}
					});
    				listPass.addListSelectionListener(new ListSelectionListener() {
    					public void valueChanged(ListSelectionEvent e) {
    						JList source = (JList)e.getSource();
    						// Update listHash
    						listHash.setSelectionInterval(source.getAnchorSelectionIndex(), source.getLeadSelectionIndex());
    					}
    				});
    				
    				// Private class to grap and handle Ctrl+c on the lists
    				class CtrlCKey extends java.awt.event.KeyAdapter {
    					public void keyReleased(java.awt.event.KeyEvent event) {
    					    if (event.getKeyCode() == java.awt.event.KeyEvent.VK_C)
    					        if (event.isControlDown()) {
    								String output = "";
    								// Special characters
    								String newline = System.getProperty("line.separator");
    								String tabulator = "\t";
    								
    								int start = listPass.getMinSelectionIndex();
    								int stop = listPass.getMaxSelectionIndex();
    								
    								// Ensure selection
    								if (start < 0 || stop < 0) {
    									showUsage(8);
    									return;
    								}
    								
    								for (int i=start; i<=stop; i++) {
    									String pass = vectorPass.elementAt(i);
    									String hash = vectorHash.elementAt(i);
    									
    									output += pass + tabulator + hash + newline;
    								}
    								
    								StringSelection string = new StringSelection(output);
    								Clipboard systemClipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
    								systemClipboard.setContents(string, new ClipboardOwner() {
    									public void lostOwnership(Clipboard clipboard, Transferable contents) {
    										// Doesn't matter
    									}
    								});
    					        }
    					}
    				}

    				
    				// Make Ctrl+c on lists take both values
    				listHash.addKeyListener(new CtrlCKey());
    				listPass.addKeyListener(new CtrlCKey());
    				
    				
    				JSplitPane splitpane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
    				splitpane.setContinuousLayout(true);
    				// Add left/right
    				splitpane.setLeftComponent(panelPass);
    				splitpane.setRightComponent(panelHash);
    				
    				JPanel content = new JPanel();
    				content.setLayout(new BorderLayout());
    				content.add(splitpane, BorderLayout.CENTER);
    				// Buttons
    				JPanel buttons = new JPanel();
    				buttons.setLayout(new FlowLayout(FlowLayout.LEFT));
    				content.add(buttons, BorderLayout.NORTH);
    				
    				btnOutputCopy = new JButton("Copy");
    				btnOutputCopy.addActionListener(new ActionListener() {
						public void actionPerformed(ActionEvent e) {
							String output = "";
							// Special characters
							String newline = System.getProperty("line.separator");
							String tabulator = "\t";
							
							int start = listPass.getMinSelectionIndex();
							int stop = listPass.getMaxSelectionIndex();
							
							// Ensure selection
							if (start < 0 || stop < 0) {
								showUsage(8);
								return;
							}
							
							for (int i=start; i<=stop; i++) {
								String pass = vectorPass.elementAt(i);
								String hash = vectorHash.elementAt(i);
								
								output += pass + tabulator + hash + newline;
							}
							
							StringSelection string = new StringSelection(output);
							Clipboard systemClipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
							systemClipboard.setContents(string, new ClipboardOwner() {
								public void lostOwnership(Clipboard clipboard, Transferable contents) {
									// Doesn't matter
								}
							});
						}
					});
    				buttons.add(btnOutputCopy);
    				
    				btnOutputSave = new JButton("Save to file");
    				btnOutputSave.addActionListener(new ActionListener() {
    					public void actionPerformed(ActionEvent e) {
    						String output = "";
							// Special characters
							String newline = System.getProperty("line.separator");
							String tabulator = "\t";
							
							// Build output content
							for (int i=0; i<vectorPass.size(); i++) {
								String pass = vectorPass.elementAt(i);
								String hash = vectorHash.elementAt(i);
								
								output += pass + tabulator + hash + newline;
							}
							
							// Show dialog
    						JFileChooser fileChooser = new JFileChooser();
    						int returnVal = fileChooser.showSaveDialog(MainWindow.this);
    						if (returnVal == JFileChooser.APPROVE_OPTION) {
    							File f = fileChooser.getSelectedFile();
    							try {
    								PrintWriter writer = new PrintWriter(new BufferedWriter(new FileWriter(f)));
    								writer.print(output);
    								writer.close();
    							} catch(IOException e2) {
    								showUsage(9);
    							}
    						}
    					}
    				});
    				buttons.add(btnOutputSave);
    				
    				JScrollPane scrollpane = new JScrollPane(content);
    				
    				// Add to tab
    				panelOutput.add(scrollpane, BorderLayout.CENTER);
                } else {
                	JPanel panelPass = new JPanel();
                	panelPass.setLayout(new BorderLayout());
                	
                	JLabel lblPass = new JLabel(pswAmount > 1 ? "Passwords" : "Password");
                	lblPass.setOpaque(true);
    				lblPass.setBackground(Color.BLACK);
    				lblPass.setForeground(Color.WHITE);
    				panelPass.add(lblPass, BorderLayout.NORTH);
    				
    				final JList listPass = new JList(vectorPass);
    				listPass.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
    				panelPass.add(listPass, BorderLayout.CENTER);
    				
    				JPanel content = new JPanel();
    				content.setLayout(new BorderLayout());
    				content.add(panelPass, BorderLayout.CENTER);
    				// Buttons
    				JPanel buttons = new JPanel();
    				buttons.setLayout(new FlowLayout(FlowLayout.LEFT));
    				content.add(buttons, BorderLayout.NORTH);
    				
    				btnOutputCopy = new JButton("Copy");
    				btnOutputCopy.addActionListener(new ActionListener() {
						public void actionPerformed(ActionEvent e) {
							String output = "";
							// Special characters
							String newline = System.getProperty("line.separator");
							String tabulator = "\t";
							
							int start = listPass.getMinSelectionIndex();
							int stop = listPass.getMaxSelectionIndex();
							
							// Ensure selection
							if (start < 0 || stop < 0) {
								showUsage(8);
								return;
							}
							
							for (int i=start; i<=stop; i++) {
								String pass = vectorPass.elementAt(i);
								
								output += pass + newline;
							}
							
							StringSelection string = new StringSelection(output);
							Clipboard systemClipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
							systemClipboard.setContents(string, new ClipboardOwner() {
								public void lostOwnership(Clipboard clipboard, Transferable contents) {
									// Doesn't matter
								}
							});
						}
					});
    				buttons.add(btnOutputCopy);
    				
    				btnOutputSave = new JButton("Save to file");
    				btnOutputSave.addActionListener(new ActionListener() {
    					public void actionPerformed(ActionEvent e) {
    						String output = "";
							// Special characters
							String newline = System.getProperty("line.separator");
							String tabulator = "\t";
							
							// Build output content
							for (String s : vectorPass) {
								output += s + newline;
							}
							
							// Show dialog
    						JFileChooser fileChooser = new JFileChooser();
    						int returnVal = fileChooser.showSaveDialog(MainWindow.this);
    						if (returnVal == JFileChooser.APPROVE_OPTION) {
    							File f = fileChooser.getSelectedFile();
    							try {
    								PrintWriter writer = new PrintWriter(new BufferedWriter(new FileWriter(f)));
    								writer.print(output);
    								writer.close();
    							} catch(IOException e2) {
    								showUsage(9);
    							}
    						}
    					}
    				});
    				buttons.add(btnOutputSave);
    				
    				JScrollPane scrollpane = new JScrollPane(content);
    				
    				// Add to tab
    				panelOutput.add(scrollpane, BorderLayout.CENTER);
                }
                tabbedPane.setEnabledAt(panelOutputIndex, true);
                tabbedPane.setSelectedIndex(panelOutputIndex);
            }
        });
	}
	
	/**
	 * Enumerable used by the changeField(...) method, determines which operation
	 * is performed when the method is called.
	 * @author Michael Dahl
	 *
	 */
	private enum ChangeOperator {
		PLUS, MINUS, EXACT;
	}
	
	/**
	 * Overloaded version with ignoreRules = false.
	 * @param f The field that contains the value
	 * @param s The slider which is assosiated with this field
	 * @param op Which operation to perform (PLUS/MINUS/EXACT)
	 * @param amount Amount to change value by
	 */
	private void changeField(JTextField f, JSlider s, ChangeOperator op, int amount) {
		changeField(f, s, op, amount, false);
	}
	
	/**
	 * Change the integer value of a JTextField, by either incrementing or decrementing
	 * by amount. If the field doesn't contain any valid number, 0 will be assumed.
	 * In the case op = EXACT, the field will be changed to exactly amount
	 * @param f The field that contains the value
	 * @param s The slider which is assosiated with this field
	 * @param op Which operation to perform (PLUS/MINUS/EXACT)
	 * @param amount Amount to change value by
	 * @param ignoreRules Whether or not to ignore rules
	 */
	private void changeField(JTextField f, JSlider s, ChangeOperator op, int amount, boolean ignoreRules) {
		int value;
		try {
			value = Integer.valueOf(f.getText());
		} catch(NumberFormatException e) {
			// Content of field, was not a number, assume value = 0
			if(f == fieldPasswordLength) {
				value = 12;
			} else {
				value = 1;
			}
		}
		// Perform change
		if (op == ChangeOperator.PLUS) {
			value += amount;
		} else if(op == ChangeOperator.MINUS) {
			value -= amount;
		} else if(op == ChangeOperator.EXACT) {
			value = amount;
		}
		
		// Ensure bounds
		if (value < 1) {
			value = 1;
		}
		
		// Update field
		f.setText(""+value);
		
		// Update slider
		if (value > s.getMaximum()) {
			s.setValue(s.getMaximum());
		} else if (value < s.getMinimum()) {
			s.setValue(s.getMinimum());
		} else {
			s.setValue(value);
		}
		
		if(!ignoreRules) {
			if(f == fieldPasswordLength) {
				// Check rules
				if(getHowManyReqRulesChecked() > value) {
		            chbReqLowCase.setSelected(false);
		            chbReqHighCase.setSelected(false);
		            chbReqDigits.setSelected(false);
		            showUsage(4);
		        }
				if(value == 1 && chbCheckRows.isSelected()) {
	                chbCheckRows.setSelected(false);
	                showUsage(7);
	            }
			}
		}
	}
	
	/**
	 * Overloaded version of changeSlider(JSlider, String, boolean) with
	 * ignoreRules = false.
	 * @param s The slider which is associated with this field
	 * @param amount Amount to set value to
	 */
	private void changeSlider(JSlider s, String amount) {
		// Call to overloaded method, with default not ignoring rules
		changeSlider(s, amount, false);
	}
	
	/**
	 * This is a wrapper method for changeField, with the difference that amount is
	 * a String type which is parsed to a number.
	 * Assumes 0 if the String can't be evaluated.
	 * @param s The slider which is associated with this field
	 * @param amount Amount to set value to
	 * @param ignoreRules Wether or not to ignore rules
	 */
	private void changeSlider(JSlider s, String amount, boolean ignoreRules) {
		int value;
		try {
			value = Integer.valueOf(amount);
		} catch(NumberFormatException e) {
			if (s == sliderPasswordLength) {
				value = 12;
			} else {
				value = 1;
			}
		}
		
		// Update slider
		if (value > s.getMaximum()) {
			s.setValue(s.getMaximum());
		} else if (value < s.getMinimum()) {
			s.setValue(s.getMinimum());
		} else {
			s.setValue(value);
		}
		
		// Check rules
		if(!ignoreRules) {
			if(s == sliderPasswordLength) {
				if(getHowManyReqRulesChecked() > value) {
		            chbReqLowCase.setSelected(false);
		            chbReqHighCase.setSelected(false);
		            chbReqDigits.setSelected(false);
		            showUsage(4);
		        }
				if(value == 1 && chbCheckRows.isSelected()) {
	                chbCheckRows.setSelected(false);
	                showUsage(7);
	            }
			}
		}
	}
	
	/**
	 * Shows a dialig message, according to the number parsed.
	 * @param num Which dialog message to display
	 */
	private void showUsage(int num) {
        String message = "";

        switch(num) {
        case 1:
            message = "You cannot require to get digits when not using the character range [0-9]!";
            break;
        case 2:
            message = "You cannot require to get low case characters when not using the character range [a-z]!";
            break;
        case 3:
            message = "You cannot require to get upper case characters when not using the character range [A-Z]!";
            break;
        case 4:
            message = "You cannot require to get upper case characters, low case characters\nand digits when trying to generate a password of length\n3 or less! Each rule requires at least one character to pass.";
            break;
        case 5:
            message = "Your chosen character set cannot generate passwords that does not only\noriginate from one row of the keyboard!\nThese are illegal examples: qwerty, 123, ghjk etc.";
            break;
        case 6:
            message = "Your chosen character set is too short!\nIf characters cannot appear more than once then the character set needs to be as long or longer than the length of passwords to generate!";
            break;
        case 7:
            message = "When characters cannot originate from only\none row then the password length cannot be 1 (one)!";
            break;
        case 8:
        	message = "You have to at least select one password from the list, before doing this";
        	break;
        case 9:
        	message = "An error occurred while trying to save. Please try again.";
        	break;
        }
        
        
        JOptionPane.showMessageDialog(this, message);
    }
	
	/**
	 * TODO Describe method
	 * @return
	 */
	private int getHowManyReqRulesChecked() {
        int res = 0;
        if(chbReqLowCase.isSelected())
            res++;
        if(chbReqHighCase.isSelected())
            res++;
        if(chbReqDigits.isSelected())
            res++;        
        return res;
    }
	
	/**
	 * TODO Describe method
	 * @param ruleBox
	 * @param rangeBox
	 */
	private void rulesCheck(JCheckBox ruleBox, JCheckBox rangeBox) {
        if(ruleBox.isSelected() && !rangeBox.isSelected()) {
            ruleBox.setSelected(false);
            int num = 1;
            if(ruleBox == chbReqDigits)
                num = 1;
            else if(ruleBox == chbReqLowCase)
                num = 2;
            else if(ruleBox == chbReqHighCase)
                num = 3;
            showUsage(num);
        } else {
        	try {
        		if(getHowManyReqRulesChecked() > Integer.valueOf(fieldPasswordLength.getText())) { 
                    ruleBox.setSelected(false);
                    showUsage(4);
        		}
        		if(Integer.valueOf(fieldPasswordLength.getText()) == 1 && chbCheckRows.isSelected()) {
                    chbCheckRows.setSelected(false);
                    showUsage(7);
                }
        	} catch(NumberFormatException e) {
        		// Not much todo
        	}
        }        
    }
}
     