package com.aconiac.apg.test;
import org.junit.*;
import static org.junit.Assert.*;
import com.aconiac.apg.domain.*;

public class TestPasswordGenerator {
    private PasswordGenerator gen;
    @Before public void setup() {
        gen = new PasswordGeneratorImpl();
    }

    @Test public void testShouldGenerate12CharPassword() {
        String psw = gen.generate(12);
        assertEquals(12, psw.length());
    }

    @Test public void testShouldGenerate16CharPassword() {
        String psw = gen.generate(16);
        assertEquals(16, psw.length());
    }

    @Test public void testShouldBeAbleToSetCharacterSet() {
        gen.setCharacterSet(PasswordGeneratorImpl.A_Through_Z_SET);
        assertEquals(PasswordGeneratorImpl.A_Through_Z_SET,
                     gen.getCharacterSet());
    }

    @Test public void testShouldProvideEntropy3With8CharPsw() {
        assertTrue(gen.checkStrength(8, PasswordGeneratorImpl.Zero_Through_Nine_SET.length()) >= 3.0);
    }

    @Test public void testShouldProvideEntropy5With8CharPsw() {
        assertTrue(gen.checkStrength(8,
                                     gen.getCharacterSet().length()) >= 5.0);
    }

    @Test public void testShouldProvideEntropy4p7With16CharPsw() {
        gen.setCharacterSet(PasswordGeneratorImpl.Zero_Through_Nine_SET+
                            PasswordGeneratorImpl.a_Through_z_SET);
        assertTrue(gen.checkStrength(16,
                                     gen.getCharacterSet().length()) >= 4.7);
    }

    @Test public void testShouldHaveOneMoreRuleWhenOneAdded() {
        assertEquals(0, gen.getRuleList().size());
        gen.addRule(new HasNumbersRule());
        assertEquals(1, gen.getRuleList().size());
    }

    @Test public void testShouldHaveOneLessRuleWhenOneRemoved() {
        Rule r = new HasNumbersRule();
        gen.addRule(r);
        assertEquals(1, gen.getRuleList().size());
        gen.removeRule(r);
        assertEquals(0, gen.getRuleList().size());
    }

    @Test public void testShouldOnlyGenerateAPasswordValidatedByTheRules() {
        Rule r = new HasNumbersRule();
        gen.addRule(r);
        String psw = gen.generate(8);
        assertTrue(r.isValid(psw));
    }

    @Test public void testShouldGetADescriptionWithEntropyGreaterThanZero() {
        assertTrue(gen.strengthDescription(20.0).length() > 0);
    }

    /** This wrapper is only required for running the old JUnit 3.8
     * graphical user interface on new JUnit 4 test cases */
    public static junit.framework.Test suite() {
        return new junit.framework.JUnit4TestAdapter(TestPasswordGenerator.class);
    } 
}
